# frozen_string_literal: true

require 'json'
require 'zlib'
require 'base64'
require 'aws-sdk-firehose'

module DasProfiler
  class Middleware
    # Internal use class to submit traces to the DasProfiler service.
    class Submitter
      SUBMISSION_INTERVAL = 60

      # Reconcile user provided options and defaults, and convert preferences
      # into usable objects.
      class Configuration
        attr_reader :options

        def initialize(options)
          @options = options
        end

        def metadata
          @metadata ||= begin
            app_name = options[:app_name] || ENV['DAS_PROFILER_APP_NAME']
            { app_name: app_name, version: options[:version],
              base_path: options[:base_path] }
          end
        end

        def client
          @client ||= begin
            access_key_id = options[:access_key_id] ||
                            ENV['DAS_PROFILER_ACCESS_KEY_ID']
            secret_access_key = options[:secret_access_key] ||
                                ENV['DAS_PROFILER_SECRET_ACCESS_KEY']

            Aws::Firehose::Client.new(
              region: 'us-east-1', access_key_id: access_key_id,
              secret_access_key: secret_access_key
            )
          end
        end
      end

      private_constant :Configuration

      # @option options [String] :access_key_id
      #   (ENV['DAS_PROFILER_ACCESS_KEY_ID']) The Das Profiler access key for
      #   your account
      # @option options [String] :secret_access_key
      #   (ENV['DAS_PROFILER_SECRET_ACCESS_KEY']) The Das Profiler secret access
      #   key for your account
      # @option options [String] :app_name (ENV['DAS_PROFILER_APP_NAME'])
      #   The name of the current application. Used to segment traces
      # @option options [String] :version The version of the current
      #   application. Using your git shortsha or similar is highly recommended
      def initialize(options)
        @configuration = Configuration.new(options)
        schedule_next_submission
        at_exit { submit }
      end

      # Attempt to submit the latest set of results from StackProf
      def submit_if_needed
        # If StackProf is currently running we hold back on the assumption that
        # this will shortly be called again after stopping StackProf. We want to
        # avoid rescheduling in that case, to minimize latency.
        return unless submission_needed? && !StackProf.running?

        schedule_next_submission if submit
      end

    private

      attr_reader :next_submission_time

      def schedule_next_submission
        @next_submission_time =
          Process.clock_gettime(Process::CLOCK_MONOTONIC) + SUBMISSION_INTERVAL
      end

      def submission_needed?
        Process.clock_gettime(Process::CLOCK_MONOTONIC) >= next_submission_time
      end

      # Submits a trace to the DasProfiler service
      # @return [nil,truthy] nil if results were not submitted, otherwise a
      #   truthy value
      def submit
        results = StackProf.results
        return unless results && !results.empty?

        @configuration.client.put_record(
          delivery_stream_name: 'dasprofiler-dev',
          record: {
            data: data_for(results)
          }
        )
      end

      def data_for(results)
        JSON.generate(
          metadata: @configuration.metadata.merge(time: Time.now.to_f),
          trace: Base64.encode64(Zlib::Deflate.deflate(JSON.generate(results)))
        )
      end
    end
  end
end
